chore: import upstream snapshot with attribution
This commit is contained in:
@@ -0,0 +1,80 @@
|
||||
// Copyright (c) 2022 PaddlePaddle Authors. All Rights Reserved.
|
||||
//
|
||||
// Licensed under the Apache License, Version 2.0 (the "License");
|
||||
// you may not use this file except in compliance with the License.
|
||||
// You may obtain a copy of the License at
|
||||
//
|
||||
// http://www.apache.org/licenses/LICENSE-2.0
|
||||
//
|
||||
// Unless required by applicable law or agreed to in writing, software
|
||||
// distributed under the License is distributed on an "AS IS" BASIS,
|
||||
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
|
||||
// See the License for the specific language governing permissions and
|
||||
// limitations under the License.
|
||||
|
||||
#include "paddle/phi/kernels/abs_kernel.h"
|
||||
|
||||
#include <algorithm>
|
||||
#include <vector>
|
||||
#include "paddle/phi/backends/gpu/gpu_context.h"
|
||||
#include "paddle/phi/core/dense_tensor.h"
|
||||
#include "paddle/phi/core/kernel_registry.h"
|
||||
#include "paddle/phi/kernels/funcs/complex_functors.h"
|
||||
#include "paddle/phi/kernels/funcs/elementwise_base.h"
|
||||
|
||||
namespace phi {
|
||||
|
||||
template <typename T, typename Enable = void>
|
||||
struct CudaAbsFunctor;
|
||||
|
||||
template <typename T>
|
||||
struct CudaAbsFunctor<T, funcs::Complex<T, dtype::Real<T>>> {
|
||||
__device__ __forceinline__ dtype::Real<T> operator()(const T x) const {
|
||||
return abs(x);
|
||||
}
|
||||
};
|
||||
|
||||
template <typename T>
|
||||
struct CudaAbsFunctor<T,
|
||||
std::enable_if_t<std::is_same<T, dtype::Real<T>>::value &&
|
||||
std::is_same<T, bfloat16>::value>> {
|
||||
__device__ __forceinline__ T operator()(const T x) const { return abs(x); }
|
||||
};
|
||||
|
||||
template <typename T>
|
||||
struct CudaAbsFunctor<T,
|
||||
std::enable_if_t<std::is_same<T, dtype::Real<T>>::value &&
|
||||
!std::is_same<T, bfloat16>::value>> {
|
||||
__device__ __forceinline__ T operator()(const T x) const {
|
||||
return std::abs(x);
|
||||
}
|
||||
};
|
||||
|
||||
template <typename T, typename Context>
|
||||
PADDLE_API void AbsKernel(const Context& dev_ctx,
|
||||
const DenseTensor& x,
|
||||
DenseTensor* out) {
|
||||
dev_ctx.template Alloc<dtype::Real<T>>(out);
|
||||
std::vector<const DenseTensor*> ins = {&x};
|
||||
std::vector<DenseTensor*> outs = {out};
|
||||
auto functor = CudaAbsFunctor<T>();
|
||||
|
||||
funcs::ElementwiseKernel<dtype::Real<T>>(dev_ctx, ins, &outs, functor);
|
||||
}
|
||||
|
||||
} // namespace phi
|
||||
|
||||
PD_REGISTER_KERNEL(abs,
|
||||
GPU,
|
||||
ALL_LAYOUT,
|
||||
phi::AbsKernel,
|
||||
float,
|
||||
double,
|
||||
int,
|
||||
int64_t,
|
||||
phi::float16,
|
||||
phi::bfloat16,
|
||||
phi::complex64,
|
||||
phi::complex128) {
|
||||
kernel->OutputAt(0).SetDataType(phi::dtype::ToReal(kernel_key.dtype()));
|
||||
}
|
||||
Reference in New Issue
Block a user